/*
 * lm63.c - driver for the National Semiconductor LM63 temperature sensor
 *          with integrated fan control
 * Copyright (C) 2004-2005  Jean Delvare <khali@linux-fr.org>
 * Based on the lm90 driver.
 *
 * The LM63 is a sensor chip made by National Semiconductor. It measures
 * two temperatures (its own and one external one) and the speed of one
 * fan, those speed it can additionally control. Complete datasheet can be
 * obtained from National's website at:
 *   http://www.national.com/pf/LM/LM63.html
 *
 * The LM63 is basically an LM86 with fan speed monitoring and control
 * capabilities added. It misses some of the LM86 features though:
 *  - No low limit for local temperature.
 *  - No critical limit for local temperature.
 *  - Critical limit for remote temperature can be changed only once. We
 *    will consider that the critical limit is read-only.
 *
 * The datasheet isn't very clear about what the tachometer reading is.
 * I had a explanation from National Semiconductor though. The two lower
 * bits of the read value have to be masked out. The value is still 16 bit
 * in width.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/jiffies.h>
#include <linux/i2c.h>
#include <linux/hwmon-sysfs.h>
#include <linux/hwmon.h>
#include <linux/err.h>
#include <linux/mutex.h>

/*
 * Addresses to scan
 * Address is fully defined internally and cannot be changed.
 */
static unsigned short normal_i2c[] = { 0x30, I2C_CLIENT_END };

/*
 * Insmod parameters
 */
I2C_CLIENT_INSMOD_1(lm63);

/*
 * The LM63 registers
 */
#define LM63_REG_FORCE_RESET		0x00
#define LM63_REG_PWM_VALUE		0x01
#define LM63_REG_COUNT_OVERFLOW		0x02
#define LM63_REG_ROTATION_DIVISOR	0x03
#define LM63_REG_TEMPERATUR		0x04
#define LM63_REG_STATUS_FLAGS		0x05

#define LM63_REG_TACHO_RPM_MSB		0x06
#define LM63_REG_TACHO_RPM_LSB		0x07

#define LM63_REG_TACHO_RAW_MSB		0x08
#define LM63_REG_TACHO_RAW_LSB		0x09

#define LM63_REG_TACHO_PRE_MSB		0x0A
#define LM63_REG_TACHO_PRE_LSB		0x0B


#define LM63_REG_MAGIC_MSB		0x20
#define LM63_REG_MAGIC_LSB		0x21

#define LM63_REG_SIGNATURE		0x22
#define LM63_REG_SIGNATURE_LENGTH	0x06

#define LM63_ROM_DESCRIPTION		0x00
#define LM63_ROM_VERSION		0x10

/*
 * The LM63 defines
 */
#define PSOC_MAGIC_VALUE		0xFEEC
#define PSOC_MAGIC_VALUE_MSB		0xFE
#define PSOC_MAGIC_VALUE_LSB		0xEC

#define PSOC_STATUS_GIE			0x80
#define PSOC_STATUS_PWM_LOCKED		0x40
#define PSOC_STATUS_WDRS 		0x20
#define PSOC_STATUS_PORS 		0x10
#define PSOC_STATUS_SLEEP 		0x08
#define PSOC_STATUS_IRESS		0x04
#define PSOC_STATUS_IRAMDIS		0x02
#define PSOC_STATUS_STOP 		0x01

/*
 * Conversions and various macros
 * For tachometer counts, the LM63 uses 16-bit values.
 * For local temperature and high limit, remote critical limit and hysteresis
 * value, it uses signed 8-bit values with LSB = 1 degree Celsius.
 * For remote temperature, low and high limits, it uses signed 11-bit values
 * with LSB = 0.125 degree Celsius, left-justified in 16-bit registers.
 */

#define FAN_FROM_REG(reg)	((reg))
#define FAN_TO_REG(val) 	((val))
#define TEMP_FROM_REG(reg)	((reg))
#define TEMP_TO_REG(val)	((val))

/*
 * Functions declaration
 */

static int lm63_attach_adapter(struct i2c_adapter *adapter);
static int lm63_detach_client(struct i2c_client *client);

static struct lm63_data *lm63_update_device(struct device *dev);

static int lm63_detect(struct i2c_adapter *adapter, int address, int kind);
static void lm63_init_client(struct i2c_client *client);

/*
 * Driver data (common to all clients)
 */

static struct i2c_driver lm63_driver = {
	.driver = {
		.name	= "DT2-Fan",
	},
	.attach_adapter	= lm63_attach_adapter,
	.detach_client	= lm63_detach_client,
};

/*
 * Client data (each client gets its own)
 */

struct lm63_data {
	struct i2c_client client;
	struct class_device *class_dev;
	struct mutex update_lock;
	char valid; /* zero until following fields are valid */
	unsigned long last_updated; /* in jiffies */

	/* registers values */
	u8 forceReset;
	u8 fanPulseWidth;
	u8 countOverflow;
	u8 rotationDivisor;
	u8 temperatur;
	u8 statusFlags;
	u16 RPM;
	u16 rawPulsesPerRound;
	u16 prePulsesPerRound;

	u16 id_magic;
	u8 signature[LM63_REG_SIGNATURE_LENGTH];
	u8 descVersion[255];
};

/*
 * Sysfs callback functions and files
 */

static ssize_t show_reset(struct device *dev, char *buf)
{
//	struct sensor_device_attribute *attr = to_sensor_dev_attr(devattr);
	struct lm63_data *data = lm63_update_device(dev);
	return sprintf(buf, "%d\n", (data->forceReset));
}

static ssize_t set_reset(struct device *dev, const char *buf, size_t count)
{
//	struct sensor_device_attribute *attr = to_sensor_dev_attr(devattr);
	struct i2c_client *client = to_i2c_client(dev);
	struct lm63_data *data = i2c_get_clientdata(client);
	long val = simple_strtol(buf, NULL, 10);

	mutex_lock(&data->update_lock);
	data->forceReset = (val);
	i2c_smbus_write_byte_data(client, LM63_REG_FORCE_RESET, data->forceReset);
	mutex_unlock(&data->update_lock);
	return count;
}

static ssize_t show_pwm(struct device *dev, char *buf)
{
	struct lm63_data *data = lm63_update_device(dev);
	return sprintf(buf, "%d\n", data->fanPulseWidth);
}

static ssize_t set_pwm(struct device *dev, const char *buf, size_t count)
{
	struct i2c_client *client = to_i2c_client(dev);
	struct lm63_data *data = i2c_get_clientdata(client);
	unsigned long val;
	
	if ((data->statusFlags) & PSOC_STATUS_PWM_LOCKED)
		return -EPERM;				 /* register is read-only */

	val = simple_strtoul(buf, NULL, 10);
	mutex_lock(&data->update_lock);
	data->fanPulseWidth = val <= 0 ? 0 :
			      val >= 255 ? 255 : (val);
	i2c_smbus_write_byte_data(client, LM63_REG_PWM_VALUE, data->fanPulseWidth);
	mutex_unlock(&data->update_lock);
	return count;
}

static ssize_t show_fan_overflow(struct device *dev, char *buf)
{
	struct lm63_data *data = lm63_update_device(dev);
	return sprintf(buf, "%d\n", data->countOverflow ? 1 : 0);
}

static ssize_t show_fan_divisor(struct device *dev, char *buf)
{
//	struct sensor_device_attribute *attr = to_sensor_dev_attr(devattr);
	struct lm63_data *data = lm63_update_device(dev);
	return sprintf(buf, "%d\n", TEMP_FROM_REG(data->rotationDivisor));
}

static ssize_t set_fan_divisor(struct device *dev, const char *buf, size_t count)
{
	struct i2c_client *client = to_i2c_client(dev);
	struct lm63_data *data = i2c_get_clientdata(client);
	long val = simple_strtol(buf, NULL, 10);

	mutex_lock(&data->update_lock);
	data->rotationDivisor = TEMP_TO_REG(val);
	i2c_smbus_write_byte_data(client, LM63_REG_ROTATION_DIVISOR, data->rotationDivisor);
	mutex_unlock(&data->update_lock);
	return count;
}

static ssize_t show_temp(struct device *dev, char *buf)
{
//	struct sensor_device_attribute *attr = to_sensor_dev_attr(devattr);
	struct lm63_data *data = lm63_update_device(dev);
	int temp;
	
	temp = (int)((data->temperatur / 3) * (data->temperatur < 150 ? 1000 : 1150));
	return sprintf(buf, "%d.%02d\n", (temp / 1000), (temp % 1000));
}

static ssize_t set_temp(struct device *dev, const char *buf, size_t count)
{
	struct i2c_client *client = to_i2c_client(dev);
	struct lm63_data *data = i2c_get_clientdata(client);
	long val = simple_strtol(buf, NULL, 10);

	mutex_lock(&data->update_lock);
	data->temperatur = TEMP_TO_REG(val);
	i2c_smbus_write_byte_data(client, LM63_REG_TEMPERATUR, data->temperatur);
	mutex_unlock(&data->update_lock);
	return count;
}

static ssize_t show_status_flags(struct device *dev, char *buf)
{
	struct lm63_data *data = lm63_update_device(dev);

	return sprintf(buf, "%d\n\nGIEnble: %d\nPWMLock: %d\nWDReset: %d\nPOReset: %d\nSLEEP  : %d\nIRESS  : %d\nIRAMDIS: %d\nSTOP   : %d\n",
			(data->statusFlags), (data->statusFlags & PSOC_STATUS_GIE ? 1 : 0), (data->statusFlags & PSOC_STATUS_PWM_LOCKED ? 1 : 0),
					     (data->statusFlags & PSOC_STATUS_WDRS ? 1 : 0), (data->statusFlags & PSOC_STATUS_PORS ? 1 : 0),
					     (data->statusFlags & PSOC_STATUS_SLEEP ? 1 : 0), (data->statusFlags & PSOC_STATUS_IRESS ? 1 : 0),
					     (data->statusFlags & PSOC_STATUS_IRAMDIS ? 1 : 0), (data->statusFlags & PSOC_STATUS_STOP ? 1 : 0));
}

static ssize_t set_status_flags(struct device *dev, const char *buf, size_t count)
{
	struct i2c_client *client = to_i2c_client(dev);
	struct lm63_data *data = i2c_get_clientdata(client);
	long val = simple_strtol(buf, NULL, 10);

	mutex_lock(&data->update_lock);
	data->statusFlags = (val);
	i2c_smbus_write_byte_data(client, LM63_REG_STATUS_FLAGS, data->statusFlags);
	mutex_unlock(&data->update_lock);
	return count;
}

static ssize_t show_rpm(struct device *dev, char *buf)
{
//	struct sensor_device_attribute *attr = to_sensor_dev_attr(devattr);
	struct lm63_data *data = lm63_update_device(dev);
	return sprintf(buf, "%d\n", FAN_FROM_REG(data->RPM));
}

static ssize_t set_rpm(struct device *dev, const char *buf, size_t count)
{
	struct i2c_client *client = to_i2c_client(dev);
	struct lm63_data *data = i2c_get_clientdata(client);
	unsigned long val = simple_strtoul(buf, NULL, 10);

	mutex_lock(&data->update_lock);
	data->RPM = FAN_TO_REG(val);
	i2c_smbus_write_byte_data(client, LM63_REG_TACHO_RPM_LSB,
				  data->RPM & 0xFF);
	i2c_smbus_write_byte_data(client, LM63_REG_TACHO_RPM_MSB,
				  data->RPM >> 8);
	mutex_unlock(&data->update_lock);
	return count;
}

static ssize_t show_raw(struct device *dev, char *buf)
{
//	struct sensor_device_attribute *attr = to_sensor_dev_attr(devattr);
	struct lm63_data *data = lm63_update_device(dev);
	return sprintf(buf, "%d\n", FAN_FROM_REG(data->rawPulsesPerRound));
}

static ssize_t set_raw(struct device *dev, const char *buf, size_t count)
{
	struct i2c_client *client = to_i2c_client(dev);
	struct lm63_data *data = i2c_get_clientdata(client);
	unsigned long val = simple_strtoul(buf, NULL, 10);

	mutex_lock(&data->update_lock);
	data->rawPulsesPerRound = FAN_TO_REG(val);
	i2c_smbus_write_byte_data(client, LM63_REG_TACHO_RAW_LSB,
				  data->rawPulsesPerRound & 0xFF);
	i2c_smbus_write_byte_data(client, LM63_REG_TACHO_RAW_MSB,
				  data->rawPulsesPerRound >> 8);
	mutex_unlock(&data->update_lock);
	return count;
}

static ssize_t show_pre(struct device *dev, char *buf)
{
//	struct sensor_device_attribute *attr = to_sensor_dev_attr(devattr);
	struct lm63_data *data = lm63_update_device(dev);
	return sprintf(buf, "%d\n", FAN_FROM_REG(data->prePulsesPerRound));
}

static ssize_t set_pre(struct device *dev, const char *buf, size_t count)
{
	struct i2c_client *client = to_i2c_client(dev);
	struct lm63_data *data = i2c_get_clientdata(client);
	unsigned long val = simple_strtoul(buf, NULL, 10);

	mutex_lock(&data->update_lock);
	data->prePulsesPerRound = FAN_TO_REG(val);
	i2c_smbus_write_byte_data(client, LM63_REG_TACHO_PRE_LSB,
				  data->prePulsesPerRound & 0xFF);
	i2c_smbus_write_byte_data(client, LM63_REG_TACHO_PRE_MSB,
				  data->prePulsesPerRound >> 8);
	mutex_unlock(&data->update_lock);
	return count;
}

static ssize_t show_magic(struct device *dev, char *buf)
{
	struct lm63_data *data = lm63_update_device(dev);
	return sprintf(buf, "%04X\n", data->id_magic);
}

static ssize_t show_signature(struct device *dev, char *buf)
{
	struct lm63_data *data = lm63_update_device(dev);
	return sprintf(buf, "%s\n", data->signature);
}

static ssize_t show_version(struct device *dev, char *buf)
{
	struct lm63_data *data = lm63_update_device(dev);
	return sprintf(buf, "%s\n", data->descVersion);
}

#if 0
static SENSOR_DEVICE_ATTR(fan1_input, S_IRUGO, show_fan, NULL, 0);
static SENSOR_DEVICE_ATTR(fan1_min, S_IWUSR | S_IRUGO, show_fan, set_fan, 1);

static DEVICE_ATTR(pwm1, S_IWUSR | S_IRUGO, show_pwm1, set_pwm1);

static DEVICE_ATTR(fan_overflow, S_IRUGO, show_fan_overflow, NULL);

static SENSOR_DEVICE_ATTR(temp1_input, S_IRUGO, show_temp, NULL, 0);
static SENSOR_DEVICE_ATTR(temp1_max, S_IWUSR | S_IRUGO, show_temp, set_temp, 1);

static SENSOR_DEVICE_ATTR(temp2_input, S_IRUGO, show_temp11, NULL, 0);
static SENSOR_DEVICE_ATTR(reset, S_IWUSR | S_IRUGO, show_reset, set_reset, 1);
static SENSOR_DEVICE_ATTR(temp2_max, S_IWUSR | S_IRUGO, show_temp11, set_temp11, 2);
static SENSOR_DEVICE_ATTR(temp2_crit, S_IRUGO, show_temp8, NULL, 2);
static DEVICE_ATTR(status_flags, S_IWUSR | S_IRUGO, show_status_flags, set_status_flags);

static DEVICE_ATTR(signature, S_IRUGO, show_signature, NULL);
#endif

static DEVICE_ATTR(reset, S_IWUSR | S_IRUGO, show_reset, set_reset);

static DEVICE_ATTR(pwm, S_IWUSR | S_IRUGO, show_pwm, set_pwm);

static DEVICE_ATTR(fan_overflow, S_IRUGO, show_fan_overflow, NULL);

static DEVICE_ATTR(fan_divisor, S_IWUSR | S_IRUGO, show_fan_divisor, set_fan_divisor);

static DEVICE_ATTR(temp_pcb, S_IRUGO, show_temp, NULL);
static DEVICE_ATTR(temp_max, S_IWUSR | S_IRUGO, show_temp, set_temp);

static DEVICE_ATTR(status_flags, S_IWUSR | S_IRUGO, show_status_flags, set_status_flags);

static DEVICE_ATTR(fan_rpm, S_IRUGO, show_rpm, NULL);
static DEVICE_ATTR(fan_min, S_IWUSR | S_IRUGO, show_rpm, set_rpm);

static DEVICE_ATTR(rotation_raw, S_IRUGO, show_raw, NULL);
static DEVICE_ATTR(rotation_min, S_IWUSR | S_IRUGO, show_raw, set_raw);

static DEVICE_ATTR(rotation_pre, S_IWUSR | S_IRUGO, show_pre, set_pre);

static DEVICE_ATTR(magic, S_IRUGO, show_magic, NULL);
static DEVICE_ATTR(signature, S_IRUGO, show_signature, NULL);
static DEVICE_ATTR(version, S_IRUGO, show_version, NULL);

/*
 * Real code
 */

static int lm63_attach_adapter(struct i2c_adapter *adapter)
{
	int rc;

	if (!(adapter->class & I2C_CLASS_HWMON))
		return 0;
	rc = i2c_probe(adapter, &addr_data, lm63_detect);

	return rc;
}

/*
 * The following function does more than just detection. If detection
 * succeeds, it also registers the new chip.
 */
static int lm63_detect(struct i2c_adapter *adapter, int address, int kind)
{
	struct i2c_client *new_client;
	struct lm63_data *data;
	int err = 0;

	if (!i2c_check_functionality(adapter, I2C_FUNC_SMBUS_BYTE_DATA))
		goto exit;

	if (!(data = kzalloc(sizeof(struct lm63_data), GFP_KERNEL))) {
		err = -ENOMEM;
		goto exit;
	}

	/* The common I2C client data is placed right before the
	   LM63-specific data. */
	new_client = &data->client;
	i2c_set_clientdata(new_client, data);
	new_client->addr = address;
	new_client->adapter = adapter;
	new_client->driver = &lm63_driver;
	new_client->flags = 0;

	/* Default to an LM63 if forced */
	if (kind == 0)
		kind = lm63;

	if (kind < 0) { /* must identify */
		u8 magic_hi, magic_lo;

		magic_hi = i2c_smbus_read_byte_data(new_client,
			 LM63_REG_MAGIC_MSB);
		magic_lo = i2c_smbus_read_byte_data(new_client,
			 LM63_REG_MAGIC_LSB);

		if (magic_hi == PSOC_MAGIC_VALUE_MSB	/* check for MAGIC value of PSoC */
		 && magic_lo == PSOC_MAGIC_VALUE_LSB) {
			kind = lm63;
		} else { /* failed */
			printk("Unsupported chip "
				"(magic_id=0x%02X%02X).\n",
				magic_hi, magic_lo);
			goto exit_free;
//			dev_dbg(&adapter->dev, "Unsupported chip "
		}
	}

	strlcpy(new_client->name, "DT2-Fan", I2C_NAME_SIZE);
	data->valid = 0;
	mutex_init(&data->update_lock);

	/* Tell the I2C layer a new client has arrived */
	if ((err = i2c_attach_client(new_client)))
		goto exit_free;

	/* Initialize the LM63 chip */
	lm63_init_client(new_client);

	/* Register sysfs hooks */
	data->class_dev = hwmon_device_register(&new_client->dev);
	if (IS_ERR(data->class_dev)) {
		err = PTR_ERR(data->class_dev);
		goto exit_detach;
	}

	device_create_file(&new_client->dev, &dev_attr_reset);
	device_create_file(&new_client->dev, &dev_attr_pwm);

	device_create_file(&new_client->dev, &dev_attr_fan_overflow);
	device_create_file(&new_client->dev, &dev_attr_fan_divisor);

	device_create_file(&new_client->dev, &dev_attr_temp_pcb);
	device_create_file(&new_client->dev, &dev_attr_temp_max);

	device_create_file(&new_client->dev, &dev_attr_status_flags);

	device_create_file(&new_client->dev, &dev_attr_fan_rpm);
	device_create_file(&new_client->dev, &dev_attr_fan_min);

	device_create_file(&new_client->dev, &dev_attr_rotation_raw);
	device_create_file(&new_client->dev, &dev_attr_rotation_min);
	device_create_file(&new_client->dev, &dev_attr_rotation_pre);

	device_create_file(&new_client->dev, &dev_attr_magic);
	device_create_file(&new_client->dev, &dev_attr_signature);
	device_create_file(&new_client->dev, &dev_attr_version);

	return 0;

exit_detach:
	i2c_detach_client(new_client);
exit_free:
	kfree(data);
exit:
	return err;
}

/* Idealy we shouldn't have to initialize anything, since the BIOS
   should have taken care of everything */
static void lm63_init_client(struct i2c_client *client)
{
	unsigned char dummy, temp, adress;
	struct lm63_data *data = i2c_get_clientdata(client);

//	data->config = i2c_smbus_read_byte_data(client, LM63_REG_CONFIG1);
//	data->config_fan = i2c_smbus_read_byte_data(client, LM63_REG_CONFIG_FAN);
#if 0
	/* Start converting if needed */
	if (data->config & 0x40) { /* standby */
		dev_dbg(&client->dev, "Switching to operational mode");
		data->config &= 0xA7;
		i2c_smbus_write_byte_data(client, LM63_REG_CONFIG1,
					  data->config);
	}
#endif
	data->fanPulseWidth = i2c_smbus_read_byte_data(client, LM63_REG_PWM_VALUE);

	data->id_magic	= (i2c_smbus_read_byte_data(client, LM63_REG_MAGIC_MSB) << 8)
			| (i2c_smbus_read_byte_data(client, LM63_REG_MAGIC_LSB));

	for (dummy = 0; dummy < LM63_REG_SIGNATURE_LENGTH; )
	{
		temp = i2c_smbus_read_byte_data(client, LM63_REG_SIGNATURE + dummy);
		data->signature[dummy++] = temp;
		if (!temp) break;
	}	

	/* Read EEPROM partition of PSoC */
	adress = client->addr;
	client->addr = adress | 0x40;

	for (dummy = 0; dummy < 255; )
	{
		temp = i2c_smbus_read_byte_data(client, LM63_ROM_DESCRIPTION + dummy);
		data->descVersion[dummy++] = temp;
		if (!temp) break;
	}	
	client->addr = adress;

	printk("%s-%s initialized.\n", data->descVersion, data->signature);

#if 0
	/* Show some debug info about the LM63 configuration */
	dev_dbg(&client->dev, "Alert/tach pin configured for %s\n",
		(data->config & 0x04) ? "tachometer input" :
		"alert output");
	dev_dbg(&client->dev, "PWM clock %s kHz, output frequency %u Hz\n",
		(data->config_fan & 0x08) ? "1.4" : "360",
		((data->config_fan & 0x08) ? 700 : 180000) / data->pwm1_freq);
	dev_dbg(&client->dev, "PWM output active %s, %s mode\n",
		(data->config_fan & 0x10) ? "low" : "high",
		(data->config_fan & 0x20) ? "manual" : "auto");
#endif
}

static int lm63_detach_client(struct i2c_client *client)
{
	struct lm63_data *data = i2c_get_clientdata(client);
	int err;

	hwmon_device_unregister(data->class_dev);

	if ((err = i2c_detach_client(client)))
		return err;

	kfree(data);
	return 0;
}

static struct lm63_data *lm63_update_device(struct device *dev)
{
	struct i2c_client *client = to_i2c_client(dev);
	struct lm63_data *data = i2c_get_clientdata(client);

	mutex_lock(&data->update_lock);

	if (time_after(jiffies, data->last_updated + HZ) || !data->valid) {
//		if (data->config & 0x04) /* tachometer enabled  */
/*		{
			data->RPM = (i2c_smbus_read_byte_data(client,
					LM63_REG_TACHO_RPM_LSB) & 0xFF)
				     | (i2c_smbus_read_byte_data(client,
					LM63_REG_TACHO_RPM_MSB) << 8);
		}
*/
		data->fanPulseWidth = i2c_smbus_read_byte_data(client, LM63_REG_PWM_VALUE);

		data->forceReset = i2c_smbus_read_byte_data(client, LM63_REG_FORCE_RESET);

		data->countOverflow = i2c_smbus_read_byte_data(client, LM63_REG_COUNT_OVERFLOW);

		data->rotationDivisor = i2c_smbus_read_byte_data(client, LM63_REG_ROTATION_DIVISOR);

		data->temperatur = i2c_smbus_read_byte_data(client, LM63_REG_TEMPERATUR);

		data->statusFlags = i2c_smbus_read_byte_data(client, LM63_REG_STATUS_FLAGS);

		data->RPM = (i2c_smbus_read_byte_data(client, LM63_REG_TACHO_RPM_MSB) << 8)
			  | (i2c_smbus_read_byte_data(client, LM63_REG_TACHO_RPM_LSB));

		data->rawPulsesPerRound = (i2c_smbus_read_byte_data(client, LM63_REG_TACHO_RAW_MSB) << 8)
					| (i2c_smbus_read_byte_data(client, LM63_REG_TACHO_RAW_LSB));

		data->prePulsesPerRound = (i2c_smbus_read_byte_data(client, LM63_REG_TACHO_PRE_MSB) << 8)
					| (i2c_smbus_read_byte_data(client, LM63_REG_TACHO_PRE_LSB));

		data->last_updated = jiffies;
		data->valid = 1;
	}

	mutex_unlock(&data->update_lock);

	return data;
}

static int __init sensors_lm63_init(void)
{
	return i2c_add_driver(&lm63_driver);
}

static void __exit sensors_lm63_exit(void)
{
	i2c_del_driver(&lm63_driver);
}

MODULE_AUTHOR("Freecom Technologies  <www.openfsg.org>");
MODULE_DESCRIPTION("DT2 Fan-Controller driver");
MODULE_LICENSE("GPL");

module_init(sensors_lm63_init);
module_exit(sensors_lm63_exit);
