/*
 * drivers/i2c/busses/i2c-mv5x8x.c
 *
 * Driver for the i2c controller on the Marvell line of host bridges for Orion
 * based on Driver of MIPS and PPC (e.g, gt642[46]0, mv643[46]0, mv644[46]0).
 *
 * Author: Mark A. Greer <mgreer@mvista.com> modified by Freecom
 *
 * 2005 (c) MontaVista, Software, Inc.  This file is licensed under
 * the terms of the GNU General Public License version 2.  This program
 * is licensed "as is" without any warranty of any kind, whether express
 * or implied.
 */
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/spinlock.h>
#include <linux/i2c.h>
#include <linux/interrupt.h>
//#include <linux/i2c-mv5x8x.h>
#include <linux/device.h>

#include <asm/io.h>

#include "i2c-mv5x8x.h"

/* Register defines */
#define	MV5X8X_I2C_REG_SLAVE_ADDR			0x00
#define	MV5X8X_I2C_REG_DATA				0x04
#define	MV5X8X_I2C_REG_CONTROL				0x08
#define	MV5X8X_I2C_REG_STATUS				0x0c
#define	MV5X8X_I2C_REG_BAUD				0x0c
#define	MV5X8X_I2C_REG_EXT_SLAVE_ADDR			0x10
#define	MV5X8X_I2C_REG_SOFT_RESET			0x1c

#define	MV5X8X_I2C_REG_CONTROL_ACK			0x00000004
#define	MV5X8X_I2C_REG_CONTROL_IFLG			0x00000008
#define	MV5X8X_I2C_REG_CONTROL_STOP			0x00000010
#define	MV5X8X_I2C_REG_CONTROL_START			0x00000020
#define	MV5X8X_I2C_REG_CONTROL_TWSIEN			0x00000040
#define	MV5X8X_I2C_REG_CONTROL_INTEN			0x00000080

/* Ctlr status values */
#define	MV5X8X_I2C_STATUS_BUS_ERR			0x00
#define	MV5X8X_I2C_STATUS_MAST_START			0x08
#define	MV5X8X_I2C_STATUS_MAST_REPEAT_START		0x10
#define	MV5X8X_I2C_STATUS_MAST_WR_ADDR_ACK		0x18
#define	MV5X8X_I2C_STATUS_MAST_WR_ADDR_NO_ACK		0x20
#define	MV5X8X_I2C_STATUS_MAST_WR_ACK			0x28
#define	MV5X8X_I2C_STATUS_MAST_WR_NO_ACK		0x30
#define	MV5X8X_I2C_STATUS_MAST_LOST_ARB 		0x38
#define	MV5X8X_I2C_STATUS_MAST_RD_ADDR_ACK		0x40
#define	MV5X8X_I2C_STATUS_MAST_RD_ADDR_NO_ACK		0x48
#define	MV5X8X_I2C_STATUS_MAST_RD_DATA_ACK		0x50
#define	MV5X8X_I2C_STATUS_MAST_RD_DATA_NO_ACK		0x58
#define	MV5X8X_I2C_STATUS_MAST_WR_ADDR_2_ACK		0xd0
#define	MV5X8X_I2C_STATUS_MAST_WR_ADDR_2_NO_ACK 	0xd8
#define	MV5X8X_I2C_STATUS_MAST_RD_ADDR_2_ACK		0xe0
#define	MV5X8X_I2C_STATUS_MAST_RD_ADDR_2_NO_ACK 	0xe8
#define	MV5X8X_I2C_STATUS_NO_STATUS			0xf8

/* Driver states */
enum {
	MV5X8X_I2C_STATE_INVALID,
	MV5X8X_I2C_STATE_IDLE,
	MV5X8X_I2C_STATE_WAITING_FOR_START_COND,
	MV5X8X_I2C_STATE_WAITING_FOR_ADDR_1_ACK,
	MV5X8X_I2C_STATE_WAITING_FOR_ADDR_2_ACK,
	MV5X8X_I2C_STATE_WAITING_FOR_SLAVE_ACK,
	MV5X8X_I2C_STATE_WAITING_FOR_SLAVE_DATA,
	MV5X8X_I2C_STATE_ABORTING,
};

/* Driver actions */
enum {
	MV5X8X_I2C_ACTION_INVALID,
	MV5X8X_I2C_ACTION_CONTINUE,
	MV5X8X_I2C_ACTION_SEND_START,
	MV5X8X_I2C_ACTION_SEND_ADDR_1,
	MV5X8X_I2C_ACTION_SEND_ADDR_2,
	MV5X8X_I2C_ACTION_SEND_DATA,
	MV5X8X_I2C_ACTION_RCV_DATA,
	MV5X8X_I2C_ACTION_RCV_DATA_STOP,
	MV5X8X_I2C_ACTION_SEND_STOP,
};

/* i2c Platform Device, Driver Data */
struct mv5x8x_i2c_pdata {
	u32	freq_m;
	u32	freq_n;
	u32	timeout;	/* In milliseconds */
	u32	retries;
	u32	frequency;
};

struct mv5x8x_i2c_data {
	int			irq;
	u32			state;
	u32			action;
	u32			cntl_bits;
	void __iomem		*reg_base;
	u32			reg_base_p;
	u32			addr1;
	u32			addr2;
	u32			bytes_left;
	u32			byte_posn;
	u32			block;
	int			rc;
	u32			freq_m;
	u32			freq_n;
	wait_queue_head_t	waitq;
	spinlock_t		lock;
	struct i2c_msg		*msg;
	struct i2c_adapter	adapter;
};

/*
 *****************************************************************************
 *
 *	Finite State Machine & Interrupt Routines
 *
 *****************************************************************************
 */
static void
mv5x8x_i2c_fsm(struct mv5x8x_i2c_data *drv_data, u32 status)
{
#ifdef DEBUG
	dev_dbg(&drv_data->adapter.dev,
			"mv5x8x_i2c_fsm: -- state: 0x%x, "
			"status: 0x%x, addr: 0x%x, flags: 0x%x\n",
			 drv_data->state, status, drv_data->msg->addr,
			 drv_data->msg->flags);
#endif
	/*
	 * If state is idle, then this is likely the remnants of an old
	 * operation that driver has given up on or the user has killed.
	 * If so, issue the stop condition and go to idle.
	 */
	if (drv_data->state == MV5X8X_I2C_STATE_IDLE) {
		drv_data->action = MV5X8X_I2C_ACTION_SEND_STOP;
		return;
	}

	if (drv_data->state == MV5X8X_I2C_STATE_ABORTING) {
		drv_data->action = MV5X8X_I2C_ACTION_SEND_STOP;
		drv_data->state = MV5X8X_I2C_STATE_IDLE;
		return;
	}

	/* The status from the ctlr [mostly] tells us what to do next */
	switch (status) {
	/* Start condition interrupt */
	case MV5X8X_I2C_STATUS_MAST_START: /* 0x08 */
	case MV5X8X_I2C_STATUS_MAST_REPEAT_START: /* 0x10 */
		drv_data->action = MV5X8X_I2C_ACTION_SEND_ADDR_1;
		drv_data->state = MV5X8X_I2C_STATE_WAITING_FOR_ADDR_1_ACK;
		break;

	/* Performing a write */
	case MV5X8X_I2C_STATUS_MAST_WR_ADDR_ACK: /* 0x18 */
		if (drv_data->msg->flags & I2C_M_TEN) {
			drv_data->action = MV5X8X_I2C_ACTION_SEND_ADDR_2;
			drv_data->state =
				MV5X8X_I2C_STATE_WAITING_FOR_ADDR_2_ACK;
			break;
		}
		/* FALLTHRU */
	case MV5X8X_I2C_STATUS_MAST_WR_ADDR_2_ACK: /* 0xd0 */
	case MV5X8X_I2C_STATUS_MAST_WR_ACK: /* 0x28 */
		if (drv_data->bytes_left > 0) {
			drv_data->action = MV5X8X_I2C_ACTION_SEND_DATA;
			drv_data->state =
				MV5X8X_I2C_STATE_WAITING_FOR_SLAVE_ACK;
			drv_data->bytes_left--;
		} else {
			drv_data->action = MV5X8X_I2C_ACTION_SEND_STOP;
			drv_data->state = MV5X8X_I2C_STATE_IDLE;
		}
		break;

	/* Performing a read */
	case MV5X8X_I2C_STATUS_MAST_RD_ADDR_ACK: /* 40 */
		if (drv_data->msg->flags & I2C_M_TEN) {
			drv_data->action = MV5X8X_I2C_ACTION_SEND_ADDR_2;
			drv_data->state =
				MV5X8X_I2C_STATE_WAITING_FOR_ADDR_2_ACK;
			break;
		}
		/* FALLTHRU */
	case MV5X8X_I2C_STATUS_MAST_RD_ADDR_2_ACK: /* 0xe0 */
		if (drv_data->bytes_left == 0) {
			drv_data->action = MV5X8X_I2C_ACTION_SEND_STOP;
			drv_data->state = MV5X8X_I2C_STATE_IDLE;
			break;
		}
		/* FALLTHRU */
	case MV5X8X_I2C_STATUS_MAST_RD_DATA_ACK: /* 0x50 */
		if (status != MV5X8X_I2C_STATUS_MAST_RD_DATA_ACK)
			drv_data->action = MV5X8X_I2C_ACTION_CONTINUE;
		else {
			drv_data->action = MV5X8X_I2C_ACTION_RCV_DATA;
			drv_data->bytes_left--;
		}
		drv_data->state = MV5X8X_I2C_STATE_WAITING_FOR_SLAVE_DATA;

		if (drv_data->bytes_left == 1)
			drv_data->cntl_bits &= ~MV5X8X_I2C_REG_CONTROL_ACK;
		break;

	case MV5X8X_I2C_STATUS_MAST_RD_DATA_NO_ACK: /* 0x58 */
		drv_data->action = MV5X8X_I2C_ACTION_RCV_DATA_STOP;
		drv_data->state = MV5X8X_I2C_STATE_IDLE;
		break;

	case MV5X8X_I2C_STATUS_MAST_WR_ADDR_NO_ACK: /* 0x20 */
	case MV5X8X_I2C_STATUS_MAST_WR_NO_ACK: /* 30 */
	case MV5X8X_I2C_STATUS_MAST_RD_ADDR_NO_ACK: /* 48 */
		/* Doesn't seem to be a device at other end */
		drv_data->action = MV5X8X_I2C_ACTION_SEND_STOP;
		drv_data->state = MV5X8X_I2C_STATE_IDLE;
		drv_data->rc = -ENODEV;
		break;

	default:
		dev_err(&drv_data->adapter.dev,
			"mv5x8x_i2c_fsm: Ctlr Error -- state: 0x%x, "
			"status: 0x%x, addr: 0x%x, flags: 0x%x\n",
			 drv_data->state, status, drv_data->msg->addr,
			 drv_data->msg->flags);
		drv_data->action = MV5X8X_I2C_ACTION_SEND_STOP;
		drv_data->state = MV5X8X_I2C_STATE_IDLE;
		drv_data->rc = -EIO;
	}
}

static void
mv5x8x_i2c_do_action(struct mv5x8x_i2c_data *drv_data)
{
#ifdef DEBUG
	dev_dbg(&drv_data->adapter.dev,
			"mv5x8x_i2c_do_action: Action: %d\n",
			drv_data->action);
#endif
	switch(drv_data->action) {
	case MV5X8X_I2C_ACTION_CONTINUE:
		writel(drv_data->cntl_bits,
			drv_data->reg_base + MV5X8X_I2C_REG_CONTROL);
		break;

	case MV5X8X_I2C_ACTION_SEND_START:
		writel(drv_data->cntl_bits | MV5X8X_I2C_REG_CONTROL_START,
			drv_data->reg_base + MV5X8X_I2C_REG_CONTROL);
		break;

	case MV5X8X_I2C_ACTION_SEND_ADDR_1:
		writel(drv_data->addr1,
			drv_data->reg_base + MV5X8X_I2C_REG_DATA);
		writel(drv_data->cntl_bits,
			drv_data->reg_base + MV5X8X_I2C_REG_CONTROL);
		break;

	case MV5X8X_I2C_ACTION_SEND_ADDR_2:
		writel(drv_data->addr2,
			drv_data->reg_base + MV5X8X_I2C_REG_DATA);
		writel(drv_data->cntl_bits,
			drv_data->reg_base + MV5X8X_I2C_REG_CONTROL);
		break;

	case MV5X8X_I2C_ACTION_SEND_DATA:
		writel(drv_data->msg->buf[drv_data->byte_posn++],
			drv_data->reg_base + MV5X8X_I2C_REG_DATA);
		writel(drv_data->cntl_bits,
			drv_data->reg_base + MV5X8X_I2C_REG_CONTROL);
		break;

	case MV5X8X_I2C_ACTION_RCV_DATA:
		drv_data->msg->buf[drv_data->byte_posn++] =
			readl(drv_data->reg_base + MV5X8X_I2C_REG_DATA);
		writel(drv_data->cntl_bits,
			drv_data->reg_base + MV5X8X_I2C_REG_CONTROL);
		break;

	case MV5X8X_I2C_ACTION_RCV_DATA_STOP:
		drv_data->msg->buf[drv_data->byte_posn++] =
			readl(drv_data->reg_base + MV5X8X_I2C_REG_DATA);
		drv_data->cntl_bits &= ~MV5X8X_I2C_REG_CONTROL_INTEN;
		writel(drv_data->cntl_bits | MV5X8X_I2C_REG_CONTROL_STOP,
			drv_data->reg_base + MV5X8X_I2C_REG_CONTROL);
		drv_data->block = 0;
		wake_up_interruptible(&drv_data->waitq);
		break;

	case MV5X8X_I2C_ACTION_INVALID:
	default:
		dev_err(&drv_data->adapter.dev,
			"mv5x8x_i2c_do_action: Invalid action: %d\n",
			drv_data->action);
		drv_data->rc = -EIO;
		/* FALLTHRU */
	case MV5X8X_I2C_ACTION_SEND_STOP:
		drv_data->cntl_bits &= ~MV5X8X_I2C_REG_CONTROL_INTEN;
		writel(drv_data->cntl_bits | MV5X8X_I2C_REG_CONTROL_STOP,
			drv_data->reg_base + MV5X8X_I2C_REG_CONTROL);
		drv_data->block = 0;
		wake_up_interruptible(&drv_data->waitq);
		break;
	}
}

static int
mv5x8x_i2c_intr(int irq, void *dev_id, struct pt_regs *regs)
{
	struct mv5x8x_i2c_data	*drv_data = dev_id;
	unsigned long	flags;
	u32		status;
	int		rc = IRQ_NONE;

	spin_lock_irqsave(&drv_data->lock, flags);
	while (readl(drv_data->reg_base + MV5X8X_I2C_REG_CONTROL) &
						MV5X8X_I2C_REG_CONTROL_IFLG) {
		status = readl(drv_data->reg_base + MV5X8X_I2C_REG_STATUS);
		mv5x8x_i2c_fsm(drv_data, status);
		mv5x8x_i2c_do_action(drv_data);
		rc = IRQ_HANDLED;
	}
	spin_unlock_irqrestore(&drv_data->lock, flags);

#ifdef DEBUG
	dev_dbg(&drv_data->adapter.dev,
			"mv5x8x_i2c_intr: Handeled: %d\n", rc);
#endif
	return rc;
}

/*
 *****************************************************************************
 *
 *	I2C Msg Execution Routines
 *
 *****************************************************************************
 */
static void
mv5x8x_i2c_prepare_for_io(struct mv5x8x_i2c_data *drv_data,
	struct i2c_msg *msg)
{
	u32	dir = 0;

	drv_data->msg = msg;
	drv_data->byte_posn = 0;
	drv_data->bytes_left = msg->len;
	drv_data->rc = 0;
	drv_data->cntl_bits = MV5X8X_I2C_REG_CONTROL_ACK |
		MV5X8X_I2C_REG_CONTROL_INTEN | MV5X8X_I2C_REG_CONTROL_TWSIEN;

	if (msg->flags & I2C_M_RD)
		dir = 1;

	if (msg->flags & I2C_M_REV_DIR_ADDR)
		dir ^= 1;

	if (msg->flags & I2C_M_TEN) {
		drv_data->addr1 = 0xf0 | (((u32)msg->addr & 0x300) >> 7) | dir;
		drv_data->addr2 = (u32)msg->addr & 0xff;
	} else {
		drv_data->addr1 = ((u32)msg->addr & 0x7f) << 1 | dir;
		drv_data->addr2 = 0;
	}
}

static void
mv5x8x_i2c_wait_for_completion(struct mv5x8x_i2c_data *drv_data)
{
	long		time_left;
	unsigned long	flags;
	char		abort = 0;

	time_left = wait_event_interruptible_timeout(drv_data->waitq,
		!drv_data->block, msecs_to_jiffies(drv_data->adapter.timeout));

	spin_lock_irqsave(&drv_data->lock, flags);
	if (!time_left) { /* Timed out */
		drv_data->rc = -ETIMEDOUT;
		abort = 1;
	} else if (time_left < 0) { /* Interrupted/Error */
		drv_data->rc = time_left; /* errno value */
		abort = 1;
	}

	if (abort && drv_data->block) {
		drv_data->state = MV5X8X_I2C_STATE_ABORTING;
		spin_unlock_irqrestore(&drv_data->lock, flags);

		time_left = wait_event_timeout(drv_data->waitq,
			!drv_data->block,
			msecs_to_jiffies(drv_data->adapter.timeout));

		if (time_left <= 0) {
			drv_data->state = MV5X8X_I2C_STATE_IDLE;
			dev_err(&drv_data->adapter.dev,
				"mv5x8x: I2C bus locked\n");
		}
	} else
		spin_unlock_irqrestore(&drv_data->lock, flags);
}

static int
mv5x8x_i2c_execute_msg(struct mv5x8x_i2c_data *drv_data, struct i2c_msg *msg)
{
	unsigned long	flags;

	spin_lock_irqsave(&drv_data->lock, flags);
	mv5x8x_i2c_prepare_for_io(drv_data, msg);

//	if (unlikely(msg->flags & I2C_M_NOSTART)) { /* Skip start/addr phases */
	if (msg->flags & I2C_M_NOSTART) { /* Skip start/addr phases */
		if (drv_data->msg->flags & I2C_M_RD) {
			/* No action to do, wait for slave to send a byte */
			drv_data->action = MV5X8X_I2C_ACTION_CONTINUE;
			drv_data->state =
				MV5X8X_I2C_STATE_WAITING_FOR_SLAVE_DATA;
		} else {
			drv_data->action = MV5X8X_I2C_ACTION_SEND_DATA;
			drv_data->state =
				MV5X8X_I2C_STATE_WAITING_FOR_SLAVE_ACK;
			drv_data->bytes_left--;
		}
	} else {
		drv_data->action = MV5X8X_I2C_ACTION_SEND_START;
		drv_data->state = MV5X8X_I2C_STATE_WAITING_FOR_START_COND;
	}

	drv_data->block = 1;
	mv5x8x_i2c_do_action(drv_data);
	spin_unlock_irqrestore(&drv_data->lock, flags);

	mv5x8x_i2c_wait_for_completion(drv_data);
#ifdef DEBUG
	dev_dbg(&drv_data->adapter.dev,
			"mv5x8x_i2c_execute_msg: Return: %d\n",
			drv_data->rc);
#endif
	return drv_data->rc;
}

/*
 *****************************************************************************
 *
 *	I2C Core Support Routines (Interface to higher level I2C code)
 *
 *****************************************************************************
 */
static u32
mv5x8x_i2c_functionality(struct i2c_adapter *adap)
{
	return I2C_FUNC_I2C | I2C_FUNC_10BIT_ADDR | I2C_FUNC_SMBUS_EMUL;
}

static int
mv5x8x_i2c_xfer(struct i2c_adapter *adap, struct i2c_msg msgs[], int num)
{
	struct mv5x8x_i2c_data *drv_data = i2c_get_adapdata(adap);
	int	i, rc;

	for (i=0; i<num; i++)
	{
#ifdef DEBUG
	dev_dbg(&drv_data->adapter.dev,
			"mv5x8x_i2c_xfer: Passing Msg[%d]\n", i);
#endif
	if ((rc = mv5x8x_i2c_execute_msg(drv_data, &msgs[i])) != 0)
			return rc;
	}

	return num;
}

static struct i2c_algorithm mv5x8x_i2c_algo = {
	.master_xfer = mv5x8x_i2c_xfer,
	.functionality = mv5x8x_i2c_functionality,
};

/*
 *****************************************************************************
 *
 *	Driver Interface & Early Init Routines
 *
 *****************************************************************************
 */
static void __devinit
mv5x8x_i2c_hw_init(struct mv5x8x_i2c_data *drv_data)
{
	writel(0, drv_data->reg_base + MV5X8X_I2C_REG_SOFT_RESET);
	writel((((drv_data->freq_m & 0xf) << 3) | (drv_data->freq_n & 0x7)),
		drv_data->reg_base + MV5X8X_I2C_REG_BAUD);
	writel(0, drv_data->reg_base + MV5X8X_I2C_REG_SLAVE_ADDR);
	writel(0, drv_data->reg_base + MV5X8X_I2C_REG_EXT_SLAVE_ADDR);
	writel(MV5X8X_I2C_REG_CONTROL_TWSIEN | MV5X8X_I2C_REG_CONTROL_STOP,
		drv_data->reg_base + MV5X8X_I2C_REG_CONTROL);
	drv_data->state = MV5X8X_I2C_STATE_IDLE;
}

static int __devinit
mv5x8x_i2c_map_regs(struct platform_device *pd,
	struct mv5x8x_i2c_data *drv_data)
{
	struct resource	*r;

	if ((r = platform_get_resource(pd, IORESOURCE_MEM, 0)) &&
		request_mem_region(r->start, MV5X8X_I2C_REG_BLOCK_SIZE,
			drv_data->adapter.name)) {

		drv_data->reg_base = ioremap(r->start,
			MV5X8X_I2C_REG_BLOCK_SIZE);
		drv_data->reg_base_p = r->start;
	} else
		return -ENOMEM;

	return 0;
}

static void __devexit
mv5x8x_i2c_unmap_regs(struct mv5x8x_i2c_data *drv_data)
{
	if (drv_data->reg_base) {
		iounmap(drv_data->reg_base);
		release_mem_region(drv_data->reg_base_p,
			MV5X8X_I2C_REG_BLOCK_SIZE);
	}

	drv_data->reg_base = NULL;
	drv_data->reg_base_p = 0;
}

/* This macro returns absolute value */
#define MV_ABS(number)  (((int)(number) < 0) ? -(int)(number) : (int)(number))
extern unsigned int mvTclk;

static int __devinit
mv5x8x_i2c_probe(struct platform_device *pd)
{
	struct mv5x8x_i2c_data		*drv_data;
	struct mv5x8x_i2c_pdata 	*pdata = pd->dev.platform_data;
	int	rc;
    	unsigned int	n, m, freq, margin, minMargin = 0xffffffff;
	unsigned int	power;
    	unsigned int	frequency = 0, freq_n = 0, freq_m = 0;


	if ((pd->id != 0) || !pdata)
		return -ENODEV;

	printk("mv5x8x: Probing I2C Controller %s\n", pd->name);

	drv_data = kmalloc(sizeof(struct mv5x8x_i2c_data), GFP_KERNEL);

	if (!drv_data)
	{
		printk("mv5x8x: Error probing I2C Controller - No MEM\n");
		return -ENOMEM;
	}

	memset(drv_data, 0, sizeof(struct mv5x8x_i2c_data));

	if (mv5x8x_i2c_map_regs(pd, drv_data)) {
		rc = -ENODEV;
		goto exit_kfree;
	}

	strncpy(drv_data->adapter.name, MV5X8X_I2C_CTLR_NAME " adapter",
		I2C_NAME_SIZE);

	if(pdata->frequency > 100000)
	{
		printk("mv5x8x: Warning TWSI frequency is too high, please use up tp 100Khz. \n");
	}

	printk("mv5x8x: - Tclk = %d, frequency = %d\n", mvTclk, pdata->frequency);
    	/* Calucalte N and M for the TWSI clock baud rate */
    	for(n = 0 ; n < 8 ; n++)
    	{
        	for(m = 0 ; m < 16 ; m++)
        	{
            		power = 2 << n; /* power = 2^(n+1) */
            		freq = mvTclk/(10*(m+1)*power);
            		margin = MV_ABS(pdata->frequency - freq);
            		if(margin < minMargin)
            		{
                		minMargin  = margin;
                		frequency   = freq;
                		freq_n     = n;
                		freq_m     = m;
            		}
        	}
	}
	printk("mv5x8x: - freq_n:%d, freq_m:%d, frequency:%d\n", freq_n , freq_m, frequency);

	init_waitqueue_head(&drv_data->waitq);
	spin_lock_init(&drv_data->lock);

	drv_data->freq_m = freq_m;
	drv_data->freq_n = freq_n;
	drv_data->irq = platform_get_irq(pd, 0);
	drv_data->adapter.id = I2C_HW_MV5X8X;
	drv_data->adapter.algo = &mv5x8x_i2c_algo;
	drv_data->adapter.owner = THIS_MODULE;
	drv_data->adapter.class = I2C_CLASS_HWMON;
	drv_data->adapter.timeout = 100;
	drv_data->adapter.retries = 3;
	platform_set_drvdata(pd, drv_data);
	i2c_set_adapdata(&drv_data->adapter, drv_data);

	if (request_irq(drv_data->irq, mv5x8x_i2c_intr, 0,
		MV5X8X_I2C_CTLR_NAME, drv_data)) {

		dev_err(&drv_data->adapter.dev, "mv5x8x: Can't register intr handler "
			"irq: %d\n", drv_data->irq);
		rc = -EINVAL;
		goto exit_unmap_regs;
	} else if ((rc = i2c_add_adapter(&drv_data->adapter)) != 0) {
		dev_err(&drv_data->adapter.dev,
			"mv5x8x: Can't add i2c adapter, rc: %d\n", -rc);
		goto exit_free_irq;
	}

	mv5x8x_i2c_hw_init(drv_data);

	return 0;

	exit_free_irq:
		free_irq(drv_data->irq, drv_data);
	exit_unmap_regs:
		mv5x8x_i2c_unmap_regs(drv_data);
	exit_kfree:
		printk("mv5x8x: Error probing I2C Controller\n");
		kfree(drv_data);
	return rc;
}

static int __devexit
mv5x8x_i2c_remove(struct platform_device *dev)
{
	struct mv5x8x_i2c_data		*drv_data = platform_get_drvdata(dev);
	int	rc;

	rc = i2c_del_adapter(&drv_data->adapter);
	free_irq(drv_data->irq, drv_data);
	mv5x8x_i2c_unmap_regs(drv_data);
	kfree(drv_data);

	return rc;
}

static struct platform_driver mv5x8x_i2c_driver = {
	.probe	= mv5x8x_i2c_probe,
	.remove	= mv5x8x_i2c_remove,
	.driver	= {
		.owner	= THIS_MODULE,
		.name	= MV5X8X_I2C_CTLR_NAME,
	},
};

static struct mv5x8x_i2c_pdata mv5x8x_i2c_pdata = {
	.freq_m			= 9,
	.freq_n			= 3,
	.timeout		= 500, /* Default timeout of 1 second */
	.retries		= 3,
	.frequency		= 150000,
};

static struct resource mv5x8x_i2c_resources[] = {
	/* Do not change the order of the IORESOURCE_MEM resources */
	[0] = {
		.name	= "mv5x8x i2c base",
		.start	= MV5X8X_I2C_OFFSET,
		.end	= MV5X8X_I2C_OFFSET + MV5X8X_I2C_REG_BLOCK_SIZE - 1,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.name	= "mv5x8x i2c irq",
		.start	= MV5X8X_IRQ_I2C,
		.end	= MV5X8X_IRQ_I2C,
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device mv5x8x_i2c_device = {
	.name		= MV5X8X_I2C_CTLR_NAME,
	.id		= 0,
	.num_resources	= ARRAY_SIZE(mv5x8x_i2c_resources),
	.resource	= mv5x8x_i2c_resources,
	.dev = {
		.platform_data = &mv5x8x_i2c_pdata,
	},
};

static int __init
mv5x8x_i2c_init(void)
{
	printk("mv5x8x: Init I2C Controller\n");
	platform_device_register(&mv5x8x_i2c_device);
	return platform_driver_register(&mv5x8x_i2c_driver);
}

static void __exit
mv5x8x_i2c_exit(void)
{
	platform_driver_unregister(&mv5x8x_i2c_driver);
	platform_device_unregister(&mv5x8x_i2c_device);
}

module_init(mv5x8x_i2c_init);
module_exit(mv5x8x_i2c_exit);

MODULE_AUTHOR("Mark A. Greer <mgreer@mvista.com>");
MODULE_DESCRIPTION("Marvell mv5x8x host bridge i2c ctlr driver");
MODULE_LICENSE("GPL");
