/*
 * IXP4XX GPIO driver LED driver
 *
 * Author: John Bowler <jbowler@acm.org>
 *
 * Copyright (c) 2006 John Bowler
 *
 * Permission is hereby granted, free of charge, to any
 * person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the
 * Software without restriction, including without
 * limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice
 * shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF
 * ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
 * SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

/*
 * reworked for FSG3 LEDS and buttons
 * by Freecom Technologies GmbH, Berlin
 */
 
 
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/spinlock.h>
#include <linux/leds.h>
#include <asm/arch/hardware.h>
#include <asm/io.h>
#include <linux/timer.h>
#include <linux/delay.h>
#include <linux/irq.h>
#include <linux/interrupt.h>

extern spinlock_t gpio_lock;

/* Up to 16 gpio lines are possible. */
#define GPIO_MAX 16
static struct ixp4xxgpioled_device {
	struct led_classdev ancestor;
	int               flags;
} ixp4xxgpioled_devices[GPIO_MAX];

static int key;
static int pressSyncJiffies, pressResetJiffies, pressUnplugJiffies;
static volatile unsigned short *ledmem_cs2, led_latch, led_blink_latch;

struct work_struct  hotplug_work;
static struct timer_list  leds_timerlist, keys_timerlist;


#ifdef CONFIG_HOTPLUG

/* Notify userspace when a key-event occurs,
 * by running '/sbin/hotplug key' with certain
 * environment variables set.
 */

static void key_run_sbin_hotplug(char *Action)
{
	char *argv[3], *envp[5], action_str[32];
	int i;

	sprintf(action_str, "ACTION=%s", Action);

        i = 0;
        argv[i++] = "/sbin/hotplug";
        argv[i++] = "key";
        argv[i] = 0;

	i = 0;
	/* minimal command environment */
	envp [i++] = "HOME=/";
	envp [i++] = "PATH=/sbin:/bin:/usr/sbin:/usr/bin";
	envp [i++] = "INTERFACE=";
	envp [i++] = action_str;
	envp [i] = 0;

        if (in_interrupt ()) {
                printk ("LEDMAN-hotplug: -- In_interrupt\n");
                return;
        }
        if (!current->fs->root) {
                printk ("LEDMAN-hotplug: -- no FS yet\n");
                return;
        }

	call_usermodehelper (argv[0], argv, envp, 0);
}

static void leds_poll(unsigned long arg)
{
	if(led_blink_latch == 0)
		return;
	  
	*ledmem_cs2 = led_latch = led_latch ^ led_blink_latch;

	/* Re-arm timer */
	leds_timerlist.expires = jiffies + HZ/2;
	add_timer(&leds_timerlist);
}

static void keys_poll(unsigned long arg)
{
	if(pressSyncJiffies == 0)
		if(pressResetJiffies == 0)
			return;

	if(pressSyncJiffies != 0) if(abs(jiffies - pressSyncJiffies) >= 4*HZ) {
		INIT_WORK(&hotplug_work, key_run_sbin_hotplug, "SHUTDOWN");
		schedule_work(&hotplug_work);
		pressSyncJiffies = 0;
		return;
	}

	if(pressResetJiffies != 0) if(abs(jiffies - pressResetJiffies) >= 4*HZ) {
		INIT_WORK(&hotplug_work, key_run_sbin_hotplug, "DEFAULTS");
		schedule_work(&hotplug_work);
		pressResetJiffies = 0;
		return;
	}

	/* Re-arm timer */
	keys_timerlist.expires = jiffies + HZ/10;
	add_timer(&keys_timerlist);
}

static irqreturn_t leds_sync_key_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	int holdkey;

	gpio_line_get(FSG3_SYNC_KEY_PIN, &holdkey);
	if(holdkey == 0) {  // key pressed ?

		if(key == 0) {  // no other key still pressed ?
			keys_timerlist.expires = jiffies + HZ/10;
			add_timer(&keys_timerlist);
		}

		key = 1;
		pressSyncJiffies = jiffies;
	}
	else {              // key released
		key = 0;
		if(pressSyncJiffies == 0)
			return IRQ_HANDLED;

		if(abs(jiffies - pressSyncJiffies) < 4*HZ) {
			INIT_WORK(&hotplug_work, key_run_sbin_hotplug, "SYNC");
			schedule_work(&hotplug_work);
		}

		pressSyncJiffies = 0;
	}

	return IRQ_HANDLED;
}

static irqreturn_t leds_reset_key_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	int holdkey;

	gpio_line_get(FSG3_RESET_KEY_PIN, &holdkey);
	if(holdkey == 0) {  // key pressed ?

		if(key == 0) {  // no other key still pressed ?
			keys_timerlist.expires = jiffies + HZ/10;
			add_timer(&keys_timerlist);
		}

		key = 1;
		pressResetJiffies = jiffies;
	}
	else {              // key released
		key = 0;
		if(pressResetJiffies == 0)
			return IRQ_HANDLED;

		if(abs(jiffies - pressResetJiffies) < 4*HZ) {
			INIT_WORK(&hotplug_work, key_run_sbin_hotplug, "REBOOT");
			schedule_work(&hotplug_work);
		}

		pressResetJiffies = 0;
	}

	return IRQ_HANDLED;
}

static irqreturn_t leds_unplug_key_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	// Mechanical key are nasty, so do some pre-cautions.
	if(abs(jiffies - pressUnplugJiffies) < HZ/2) {
		pressUnplugJiffies = jiffies;
		return IRQ_HANDLED;
	}
	pressResetJiffies = jiffies;

	if(led_latch & 0x10)   // test USB LED
		INIT_WORK(&hotplug_work, key_run_sbin_hotplug, "UNPLUG");
	else
		INIT_WORK(&hotplug_work, key_run_sbin_hotplug, "COLDPLUG");
	schedule_work(&hotplug_work);
	return IRQ_HANDLED;
}
#endif   /* CONFIG_HOTPLUG */

void ixp4xxgpioled_all_off(void)
{
	// all LEDs off and EL ring always off
	*ledmem_cs2 = led_latch = 0xff9f;
}

void ixp4xxgpioled_brightness_set(struct led_classdev *pled,
				enum led_brightness value)
{
	const struct ixp4xxgpioled_device *const ixp4xx_dev =
		container_of(pled, struct ixp4xxgpioled_device, ancestor);
	const u32 led_no = ixp4xx_dev - ixp4xxgpioled_devices;

	if (led_no < GPIO_MAX && ixp4xx_dev->ancestor.name != 0) {
		if(value == 2) {
			if(led_blink_latch == 0) {
				leds_timerlist.expires = jiffies + HZ/2;
				add_timer(&leds_timerlist);
			}
			led_blink_latch |= ixp4xx_dev->flags;
			*ledmem_cs2 = led_latch = led_latch & (~ixp4xx_dev->flags);
		}
		else {
			led_blink_latch &= (~ixp4xx_dev->flags);
			if(value == 0)
				*ledmem_cs2 = led_latch = led_latch | ixp4xx_dev->flags;
			else
				*ledmem_cs2 = led_latch = led_latch & (~ixp4xx_dev->flags);
		}
	}
}

/* LEDs are described in resources, the following iterates over the valid
 * LED resources.
 */
#define for_all_leds(i, pdev) \
	for (i=0; i<pdev->num_resources; ++i) \
		if (pdev->resource[i].start < GPIO_MAX && \
			pdev->resource[i].name != 0)

/* The following applies 'operation' to each LED from the given platform,
 * the function always returns 0 to allow tail call elimination.
 */
static int apply_to_all_leds(struct platform_device *pdev,
	void (*operation)(struct led_classdev *pled))
{
	int i;

	for_all_leds(i, pdev)
		operation(&ixp4xxgpioled_devices[pdev->resource[i].start].ancestor);
	return 0;
}

#ifdef CONFIG_PM
static int ixp4xxgpioled_suspend(struct platform_device *pdev,
				pm_message_t state)
{
	return apply_to_all_leds(pdev, led_classdev_suspend);
}

static int ixp4xxgpioled_resume(struct platform_device *pdev)
{
	return apply_to_all_leds(pdev, led_classdev_resume);
}
#endif

static void ixp4xxgpioled_remove_one_led(struct led_classdev *pled)
{
	led_classdev_unregister(pled);
	pled->name = 0;
}

static int ixp4xxgpioled_remove(struct platform_device *pdev)
{
	return apply_to_all_leds(pdev, ixp4xxgpioled_remove_one_led);
}

static int ixp4xxgpioled_probe(struct platform_device *pdev)
{
	/* The board level has to tell the driver where the
	 * LEDs are connected - there is no way to find out
	 * electrically.  It must also say whether the GPIO
	 * lines are active high or active low.
	 *
	 * To do this read the num_resources (the number of
	 * LEDs) and the struct resource (the data for each
	 * LED).  The name comes from the resource, and it
	 * isn't copied.
	 */
	int i;

	/* Map the LED chip select address space */
	ledmem_cs2 = (volatile unsigned short *) ioremap(IXP425_EXP_BUS_CS2_BASE_PHYS, 512);
	*ledmem_cs2 = led_latch = 0xffff;
	led_blink_latch = 0x80;    // sync LED blinking

#ifdef CONFIG_HOTPLUG
	/* Configure interrupt input for SYNC switch */
	set_irq_type(IRQ_IXP4XX_GPIO4, IRQT_BOTHEDGE);
	if (request_irq(IRQ_IXP4XX_GPIO4, leds_sync_key_interrupt, IRQF_DISABLED, "Sync", NULL) < 0)
		printk("LEDs: failed to register IRQ%d for SYNC switch\n", IRQ_IXP4XX_GPIO4);
	else
		printk("LEDs: registered SYNC switch on IRQ%d\n", IRQ_IXP4XX_GPIO4);

	/* Configure interrupt input for RESET switch */
	set_irq_type(IRQ_IXP4XX_GPIO9, IRQT_BOTHEDGE);
	if (request_irq(IRQ_IXP4XX_GPIO9, leds_reset_key_interrupt, IRQF_DISABLED, "Reset", NULL) < 0)
		printk("LEDs: failed to register IRQ%d for RESET switch\n", IRQ_IXP4XX_GPIO9);
	else
		printk("LEDs: registered RESET switch on IRQ%d\n", IRQ_IXP4XX_GPIO9);

	/* Configure interrupt input for UNPLUG switch */
	set_irq_type(IRQ_IXP4XX_GPIO10, IRQT_FALLING);
	if (request_irq(IRQ_IXP4XX_GPIO10, leds_unplug_key_interrupt, IRQF_DISABLED, "Unplug", NULL) < 0)
		printk("LEDs: failed to register IRQ%d for UNPLUG switch\n", IRQ_IXP4XX_GPIO10);
	else
		printk("LEDs: registered UNPLUG switch on IRQ%d\n", IRQ_IXP4XX_GPIO10);
#endif


	init_timer(&leds_timerlist);
	leds_timerlist.expires = jiffies + HZ/2;
	leds_timerlist.function = leds_poll;
	leds_timerlist.data = 0;
	add_timer(&leds_timerlist);

#ifdef CONFIG_HOTPLUG
	init_timer(&keys_timerlist);
	keys_timerlist.expires = jiffies + HZ/2;
	keys_timerlist.function = keys_poll;
	keys_timerlist.data = 0;
	add_timer(&keys_timerlist);

	// at start-up one IRQ is released !?! So reset everything.
	key = 0;
	pressSyncJiffies = pressResetJiffies = pressUnplugJiffies = 0;
#endif


	for_all_leds(i, pdev) {
		const u8 gpio_pin = pdev->resource[i].start;
		int      rc;

		if (ixp4xxgpioled_devices[gpio_pin].ancestor.name == 0) {

			ixp4xxgpioled_devices[gpio_pin].flags =
				pdev->resource[i].flags & IORESOURCE_BITS;

			ixp4xxgpioled_devices[gpio_pin].ancestor.name =
				pdev->resource[i].name;

			/* This is how a board manufacturer makes the LED
			 * come on on reset - the GPIO line will be high, so
			 * make the LED light when the line is low...
			 */
			if (ixp4xxgpioled_devices[gpio_pin].flags != IXP4XX_GPIO_LOW)
				ixp4xxgpioled_devices[gpio_pin].ancestor.brightness = 100;
			else
				ixp4xxgpioled_devices[gpio_pin].ancestor.brightness = 0;

			ixp4xxgpioled_devices[gpio_pin].ancestor.flags = 0;

			ixp4xxgpioled_devices[gpio_pin].ancestor.brightness_set =
				ixp4xxgpioled_brightness_set;

			ixp4xxgpioled_devices[gpio_pin].ancestor.default_trigger = 0;
		}

		rc = led_classdev_register(&pdev->dev,
				&ixp4xxgpioled_devices[gpio_pin].ancestor);
		if (rc < 0) {
			ixp4xxgpioled_devices[gpio_pin].ancestor.name = 0;
			ixp4xxgpioled_remove(pdev);
			return rc;
		}
	}

	return 0;
}

static struct platform_driver ixp4xxgpioled_driver = {
	.probe   = ixp4xxgpioled_probe,
	.remove  = ixp4xxgpioled_remove,
#ifdef CONFIG_PM
	.suspend = ixp4xxgpioled_suspend,
	.resume  = ixp4xxgpioled_resume,
#endif
	.driver  = {
		.name = "IXP4XX-GPIO-LED",
	},
};

static int __init ixp4xxgpioled_init(void)
{
	return platform_driver_register(&ixp4xxgpioled_driver);
}

static void __exit ixp4xxgpioled_exit(void)
{
	free_irq(FSG3_SYNC_KEY_PIN, NULL);
	free_irq(FSG3_RESET_KEY_PIN, NULL);
	free_irq(FSG3_UNPLUG_KEY_PIN, NULL);

	platform_driver_unregister(&ixp4xxgpioled_driver);
}

module_init(ixp4xxgpioled_init);
module_exit(ixp4xxgpioled_exit);

MODULE_AUTHOR("John Bowler <jbowler@acm.org>");
MODULE_DESCRIPTION("IXP4XX GPIO LED driver");
MODULE_LICENSE("Dual MIT/GPL");
